/*** ^^A -*-C++-*- **********************************************/
/*      extractpars.mm             28.11.2008                   */
/****************************************************************/
/*      Short Description :                                     */
/****************************************************************/
/*      Keywords :                                              */
/****************************************************************/
/*      Description/Usage :                                     */
/****************************************************************/
/*      Author(s) :                                             */
/*      Name            : Wolfgang Bermel                       */
/*      Organisation    : Bruker BioSpin GmbH                   */
/*      Email           : wolfgang.bermel@bruker-biospin.de     */
/****************************************************************/
/*      Name            Date    Modification:                   */
/*      ber             081128  created from prodec_sort_7      */
/*      ber             090810  nuc setting changed for observe */
/*                                 dimension                    */
/****************************************************************/
/*
  $Id: $
*/


AUERR = extractpars(curdat);
QUIT


/***** buildpath function *****/

static void buildpatha(char* path, const char* name1, int expno1, const char* file)
{
    char save_name[PATH_MAX];
    int save_expno = expno;
    (void)strcpy(save_name, name);
    (void)strcpy(name, name1);
    expno = expno1;

    (void)strcpy(path, ACQUPATH(file));
    (void)strcpy(name, save_name);
    expno = save_expno;
}


/***** begin program *****/

int extractpars(const char* curdat)
{

#define MAXDIM 8


    char outfile1[PATH_MAX];
    char text[PATH_MAX];
    char text2[8196];
    char nuc[10][MAXDIM];
    char nuc_list[10*200];
    char pulprog[PATH_MAX];

    char *charpoi;

    double inf[MAXDIM], sw[MAXDIM], o1p[MAXDIM];
    double grpdly = 0, phc0, phc1;

    float cnst, dw, in_f;

    int td[MAXDIM], fnmode[MAXDIM];
    int counter2, counter3;
    int nuc_type = 0;
    int byteorder, parmode;
    int curexpno, curprocno;

    FILE *fpout;


/***** get dataset *****/

    curexpno = expno;
    curprocno = procno;


    FETCHPAR("PARMODE",&parmode);
    if ( parmode > 4 )
    {
	STOPMSG("dimensionality higher than 5D currently not supported");
    }


    FETCHPARS("TD",&td[parmode+1]);

    FETCHPARS("BYTORDA",&byteorder)


    FETCHPARS("GRPDLY", &grpdly);
    FETCHPARS("DW", &dw);

    phc0 = - fmod(grpdly * 180.0, 360.0);
    phc1 = grpdly * 360.0;

    FETCHPARS("PULPROG", pulprog);


/***** read nuc *****/

    for ( counter2 = 1; counter2 <= parmode+1; counter2++ )
    {
	nuc_type = 0;

        FETCHPARNS(counter2, "TD",&td[counter2]);

        FETCHPARNS(counter2, "FnMODE",&fnmode[counter2]);

	FETCHPARNS(counter2, "NUC1", nuc[counter2]);

	if (counter2 <= parmode )
	{
	    if ( strcmp (nuc[counter2], "1H") == 0 )
	    {
		for (;;)
		{
		    (void)sprintf(text2,"Nucleus %s found in F%d dimension of\n\n"
				  "%s\n\n"
				  "specify type of nucleus:\n"
				  "\t(0)\tHall\n"
				  "\t(1)\tHN\n"
				  "\t(2)\tHa\n"
				  "\t(3)\tHab\n"
				  "\t(4)\tHali\n\n"
				  "enter selection: ", nuc[counter2], counter2, pulprog);

		    GETINT(text2, nuc_type);

		    if ( nuc_type >= 0 && nuc_type <= 4 )
			break;
		}

		if ( nuc_type == 0 )
		    (void)strcpy(nuc[counter2], "Hall");

		if ( nuc_type == 1 )
		    (void)strcpy(nuc[counter2], "HN");

		if ( nuc_type == 2 )
		    (void)strcpy(nuc[counter2], "Ha");

		if ( nuc_type == 3 )
 		    (void)strcpy(nuc[counter2], "Hab");

		if ( nuc_type == 4 )
		    (void)strcpy(nuc[counter2], "Hali");
	    }

	    if ( strcmp (nuc[counter2], "13C") == 0 )
	    {
		for(;;)
		{
		    (void)sprintf(text2,"Nucleus %s found in F%d dimension of\n\n"
				  "%s\n\n"
				  "specify type of nucleus:\n"
				  "\t(0)\tCO\n"
				  "\t(1)\tCa\n"
				  "\t(2)\tCab\n"
				  "\t(3)\tCali\n\n"
				  "enter selection: ", nuc[counter2], counter2, pulprog);

		    GETINT(text2, nuc_type);

		    if ( nuc_type >= 0 && nuc_type <= 3 )
			break;
		}

		if ( nuc_type == 0 )
		    (void)strcpy(nuc[counter2], "CO");

		if ( nuc_type == 1 )
		    (void)strcpy(nuc[counter2], "Ca");

		if ( nuc_type == 2 )
		    (void)strcpy(nuc[counter2], "Cab");

		if ( nuc_type == 3 )
		    (void)strcpy(nuc[counter2], "Cali");
	    }

	    if ( strcmp (nuc[counter2], "15N") == 0 )
		(void)strcpy(nuc[counter2], "N");
	}

        (void)strcpy(nuc[parmode+1], "Hall");

/* get increment and SW */

	if  ( counter2 <= parmode )
	{
	    FETCHPARNS(counter2, "IN_F", &in_f);
	    inf[counter2] = (double)in_f / 1000000.0;
	}

	else
	{
	    FETCHPARS("DW", &dw);
	    inf[counter2] = 2.0/1000000.0 * (double)dw;
	}


	FETCHPARNS(counter2, "SW", &sw[counter2]);


/* get carrier frequency */

	if ( strcmp (nuc[counter2], "CO") == 0 )
	{
	    FETCHPARNS(parmode+1, "CNST 21", &cnst);
	    o1p[counter2] = (double)cnst;
	}

	else if ( strcmp (nuc[counter2], "Ca") == 0 )
	{
	    FETCHPARNS(parmode+1, "CNST 22", &cnst);
	    o1p[counter2] = (double)cnst;
	}

	else if ( strcmp (nuc[counter2], "Cab") == 0  || strcmp (nuc[counter2], "Cali") == 0 )
	{
	    FETCHPARNS(parmode+1, "CNST 23", &cnst);
	    o1p[counter2] = (double)cnst;
	}

	else
	{
	    FETCHPARNS(counter2, "O1P", &o1p[counter2]);
	}
    }


/***** create text string for nuc *****/

    charpoi = nuc_list;


    for ( counter2 = parmode+1; counter2 > 0; counter2-- )
    {
	charpoi += sprintf(charpoi, " %s", nuc[counter2]);

	if ( counter2 > 1 )
	    charpoi += sprintf(charpoi, ",");
	else
	    charpoi += sprintf(charpoi, "\n");
    }



/****     print protocol file     *****/

    buildpatha(outfile1, name, curexpno, "extractpars.txt");

    fpout = fopen(outfile1,"wt");

    if ( fpout == NULL )
    {
	(void)sprintf(text,"cannot open output file:\n%s",outfile1);
	STOPMSG(text);
    }


    (void)fprintf(fpout, "FORMAT= Bruker\n");

    (void)fprintf(fpout, "NUCLEI=%s", nuc_list);

    (void)fprintf(fpout, "SW_ppm=");
    (void)fprintf(fpout, " (%.3f),", sw[parmode+1]);
    for ( counter3 = parmode; counter3 > 0; counter3-- )
    {
       (void)fprintf(fpout, " %.3f", sw[counter3]);

	if ( counter3 > 1 )
	    (void)fprintf(fpout, ",");
	else
	    (void)fprintf(fpout, "\n");
    }

    (void)fprintf(fpout, "SW_hz=");
    (void)fprintf(fpout, " (%.3f),", 1.0/inf[parmode+1]);
    for ( counter3 = parmode; counter3 > 0; counter3-- )
    {
       (void)fprintf(fpout, " %.3f", 1.0/inf[counter3]);

	if ( counter3 > 1 )
	    (void)fprintf(fpout, ",");
	else
	    (void)fprintf(fpout, "\n");
    }

    (void)fprintf(fpout, "O1_ppm=");
    (void)fprintf(fpout, " (%.3f),", o1p[parmode+1]);
    for ( counter3 = parmode; counter3 > 0; counter3-- )
    {
        (void)fprintf(fpout, " %.3f", o1p[counter3]);

	if ( counter3 > 1 )
	    (void)fprintf(fpout, ",");
	else
	    (void)fprintf(fpout, "\n");
    }

    (void)fprintf(fpout, "SIZE=");
    (void)fprintf(fpout, " (%d),", td[parmode+1]/2);
    for ( counter3 = parmode; counter3 > 0; counter3-- )
    {
        (void)fprintf(fpout, " %d", td[counter3]/2);

	if ( counter3 > 1 )
	    (void)fprintf(fpout, ",");
	else
	    (void)fprintf(fpout, "\n");
    }

    (void)fprintf(fpout, "FnMODE=");
    (void)fprintf(fpout, " (%d),", fnmode[parmode+1]);
    for ( counter3 = parmode; counter3 > 0; counter3-- )
    {
        (void)fprintf(fpout, " %d", fnmode[counter3]);

	if ( counter3 > 1 )
	    (void)fprintf(fpout, ",");
	else
	    (void)fprintf(fpout, "\n");
    }


    (void)fprintf(fpout, "GRPDLY=");
    (void)fprintf(fpout, " %.5f: zero: %.5f, first: %.5f\n", grpdly, phc0, phc1);

    (void)fprintf(fpout, "PULSEPROGRAM=");
    (void)fprintf(fpout, " %s\n", pulprog);


    (void)fprintf(fpout, "BYTEORDER=");
    (void)fprintf(fpout, " %d\n", byteorder);


    (void)fclose(fpout);


    Proc_err(ERROPT_AK_NO, "extractpars finished");


    return 0;
}
