/*****************************************************************/
/*      based on nls2_setup_4.be           07.09.2010            */
/*		  updated by M. Mobli 20140911 adding jittered sampling    */
/*			and poisson distribution approximation									 */
/*****************************************************************/
/*      Short Description :                                      */
/*****************************************************************/
/*      Keywords :                                               */
/*****************************************************************/
/*      Description/Usage :                                      */
/*****************************************************************/
/*      Author(s) :                                              */
/*      Name            : Wolfgang Bermel                        */
/*      Organisation    : Bruker BioSpin GmbH                    */
/*      Email           : wolfgang.bermel@bruker-biospin.de      */
/*****************************************************************/
/*      Name            Date    Modification:                    */
/*      ber             081030  created                          */
/*      ber             100818  modified for TS3.0               */
/*                                 sort option added (sched3d3)  */
/*      ber             100907  handling for 2D changed          */
/*****************************************************************/
/*
$Id: $
*/

#define MAXDIM 8


char infile[PATH_MAX], outfile[PATH_MAX], outfile2[PATH_MAX];
char result[PATH_MAX];
char input[PATH_MAX], path[PATH_MAX];
char nuslist[PATH_MAX];
char ct_inp[4][MAXDIM];
char sw_inp[4][MAXDIM];
char yon[4];
char sparse[32];


char *cp, *ep;

const char* args[4];

double lw[MAXDIM], j_coup[MAXDIM] ;

float swh_a, swh[MAXDIM];

int parmode, counter;
int td[MAXDIM], td_sparse, td_max,sinw[MAXDIM];
int sort_flag = 0;
int jit_flag = 0;
int randsd = 1231;
long int tval[MAXDIM];


FILE *fpi, *fpo;

/***** get dataset *****/

GETCURDATA


FETCHPAR("PARMODE",&parmode)

if ( parmode > 3 )
   STOPMSG("dimensionality higher than 4D currently not supported");

if ( parmode == 0 )
   STOPMSG("setup does not work for 1D data");


/***** initialize parameters *****/

for ( counter = 1; counter <= parmode  ;counter++ )
   {
   tval[counter] = 0;

   (void)strcpy(ct_inp[counter],"n");
   (void)strcpy(sw_inp[counter],"n");
   lw[counter] = 1.0;
   sinw[counter]=-1;

   j_coup[counter] = 0.0;
   }


/*N15 mostly CT */
(void)strcpy(ct_inp[parmode],"y");


(void)strcpy(sparse,"y");

td_sparse = 100;

(void)strcpy(yon,"y");

/***** get parameters *****/

for ( counter = 1; counter <= parmode  ;counter++ )
   {
   FETCHPARN(counter, "TD", &td[counter])
   FETCHPARN(counter, "SWH", &swh[counter])
   }

FETCHPAR("SWH",&swh_a)


/***** get info *****/

(void)sprintf(input,"nls2_setup");

td_max = td[1]/2;

for ( counter = 2; counter <= parmode  ;counter++ )
   {
   td_max *= td[counter]/2;
   }

for (;;)
   {
   GETINT("Number of sample points to collect? \n", td_sparse);

   if ( td_sparse >= 0 && td_sparse <= td_max )
      break;
   }



for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)sprintf(text,"constant time in t%d (y or n): \n", counter);
   GETSTRING(text, ct_inp[counter]);

   if ( strcmp( ct_inp[counter], "y" ) != 0 )
      {
      lw[counter] = 30.0;
      (void)sprintf(text,"linewidth in t%d (in Hz): \n", counter);
      GETDOUBLE(text, lw[counter]);
      

      }
   		(void)sprintf(text,"Add sine weighting? t%d (y or n): \n", counter);
   		GETSTRING(text, sw_inp[counter]);

   			if ( strcmp( sw_inp[counter], "n" ) != 0 )
      	{
      	sinw[counter] = 0;
      	(void)sprintf(text,"Sine-weight parameter in t%d (0 approximates sine weighted poisson-gap distribution): \n", counter);
      	GETINT(text, sinw[counter]);
      	}      
      (void)sprintf(text,"J coupling in t%d (in Hz): \n", counter);
      GETDOUBLE(text, j_coup[counter]);
      
   }


for (;;)
   {
   GETINT("Sort sampling points according to outermost loop? \n\n"
          "   (0)   no\n\n"
          "   (1)   yes\n", sort_flag);
    if ( sort_flag == 0 || sort_flag == 1 )
      break;
   }
for (;;)
   {
   GETINT("Distributed samples using jittered sampling? \n\n"
          "   (0)   no\n\n"
          "   (1)   yes with weighted samples\n\n"
          "   (2)   yes with random samples\n", jit_flag);
    if ( jit_flag == 0 || jit_flag == 1 || jit_flag == 2 )
      break;
   }
 
     GETINT("Enter random seed for schedule? \n\n" , randsd);

/***** check and store parameter file *****/

(void)sprintf(outfile,"%s.in", ACQUPATH(input) );

fpo = fopen(outfile,"wt");

if ( fpo == NULL )
   {
   (void)sprintf(text,"cannot open output file:\n%s",outfile);
   STOPMSG(text);
   }


(void)fprintf(fpo, "RAND_SEED\t%d\n", randsd);
(void)fprintf(fpo, "NDIM\t%d\n", parmode);
(void)fprintf(fpo, "NPTS_SUB\t%d\n", td_sparse);

(void)fprintf(fpo, "NPTS\t");
for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)fprintf(fpo, "%d ", td[counter]/2);
   }
(void)fprintf(fpo, "\n");

(void)fprintf(fpo, "SW\t");
for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)fprintf(fpo, "%.3f ", swh[counter]);
   }
(void)fprintf(fpo, "\n");

(void)fprintf(fpo, "LW\t");
for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)fprintf(fpo, "%.6f ", lw[counter]);
   }
(void)fprintf(fpo, "\n");

(void)fprintf(fpo, "Jsp\t ");
for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)fprintf(fpo, "0 ");
   }
(void)fprintf(fpo, "\n");

(void)fprintf(fpo, "SINE\t ");
for ( counter = 1; counter <= parmode  ;counter++ )
   {
   (void)fprintf(fpo, "%d ", sinw[counter]);
   }
(void)fprintf(fpo, "\n");

(void)fprintf(fpo, "FORCE\n");
//(void)fprintf(fpo, "FORMAT\tDEFAULT\n");
if (sort_flag == 1 )
{
(void)fprintf(fpo, "SORT\n");
}
if (jit_flag == 1 )
{
(void)fprintf(fpo, "Jit\t%d\n",jit_flag+1);
}
if (jit_flag == 2 )
{
(void)fprintf(fpo, "Jit\t%d\n",jit_flag+1);
}

(void)fclose(fpo);


/***** check and execute sched3d *****/

(void)sprintf(path,"%s/bin/sched3d5.exe", PathXWinNMRProg() );

if (access(path, F_OK))
   {
   (void)sprintf(text, "file %s does not exist", path);
   STOPMSG(text);
   }


(void)strcpy(path, ACQUPATH(""));

(void)chdir(path);


(void)sprintf(result,"%s.out", ACQUPATH(input) );


(void)sprintf(path,"%s/bin/sched3d5.exe", PathXWinNMRProg() );

/* not possible to suppress output window
args[0] = path;
args[1] = outfile;
args[2] = result;
args[3] = 0;

if ( ExecProgram(args, 1) < 0 )
   {
   STOP;
   }
*/

(void)sprintf(outfile2,"%s.log", ACQUPATH(input) );


(void)sprintf(text, "%s %s %s > %s", path, outfile, result, outfile2);
(void)system(text);



/***** check sched3d result *****/

(void)sprintf(infile,"%s.out", ACQUPATH(input) );


fpi = fopen(infile,"rt");

if ( fpi == NULL )
   {
   (void)sprintf(text,"cannot open input file:\n%s",infile);
   STOPMSG(text);
   }


/***** check output files *****/

(void)sprintf(nuslist,"nuslist_%d", expno);

(void)sprintf(outfile,"%s/stan/nmr/lists/vc/%s", PathXWinNMRExp(), nuslist);

fpo = fopen(outfile,"wt");

if ( fpo == NULL )
   {
   (void)sprintf(text,"cannot open output file:\n%s",outfile);
   }


/***** split points file *****/

while( fgets(text, sizeof(text), fpi) != NULL )
   {
   cp = text;

   while( isspace(*cp) )
      {
      cp++;
      }

   if ( *cp )
      {
      tval[1] = strtol(cp, &ep, 10);

      if ( cp == ep )
         {
         (void)sprintf(path,"inconsistent file content for input file (first entry):\n%s",infile);
         STOPMSG(path);
         }


      if ( parmode >= 2 )
         {
         cp = ep;

         tval[2] = strtol(cp, &ep, 10);

         if ( cp == ep )
            {
            (void)sprintf(path,"inconsistent file content for input file (second entry):\n%s",infile);
            STOPMSG(path);
            }
	 }

      if ( parmode >= 3 )
         {
         cp = ep;

         tval[3] = strtol(cp, &ep, 10);

         if ( cp == ep )
            {
            (void)sprintf(path,"inconsistent file content for input file (third entry):\n%s",infile);
            STOPMSG(path);
            }
	 }


      if ( parmode >= 3 )
         (void)fprintf(fpo,"%ld\t", tval[3]-1);

      if ( parmode >= 2 )
         (void)fprintf(fpo,"%ld\t", tval[2]-1);

      (void)fprintf(fpo,"%ld\n", tval[1]-1);
      }
   }


(void)fclose(fpi);
(void)fclose(fpo);


/***** store parameters *****/

STOREPAR("NUSLIST", nuslist)


STOREPAR("L 3", td_sparse*2 );

if ( parmode >= 2 )
   STOREPAR("L 13", 2 );

if ( parmode >= 3 )
   STOREPAR("L 23", 2 );


QUITMSG("nls2_setup finished")
